// Schalter
boolean anlagePowerSchalter = false;
boolean anlageScharfSchalter = false;
boolean resetTaster = false;
boolean testTaster = false;
boolean alterZustand = false;
boolean mainPowerInput = false;

//arduino Pins
int sensor = A0;
int testPin = 2;
int anlagePowerPin = 3;
int summer = 4;
int ledFeuerRed = 5;
int ledAnlageScharf = 6;
int ledAnlagePower = 7;
int ledArduinoPower = 8;
int spannungSensorPower = 9;
int anlageScharfPin = 10;
int resetPin = 11;
int mainPowerPin = 12;
int mainPower = 13;

//globale Variablen für Zustände
boolean testDurchgefuhrt = false;
boolean runTest = false;
boolean alarm_blink = false;
int sensorWert;

/*
   initiales Setup vor dem Programmstart
*/
void setup() {
  //Set Input PinMode
  pinMode(sensor, INPUT);
  pinMode(anlageScharfPin, INPUT);
  pinMode(resetPin, INPUT);
  pinMode(testPin, INPUT);
  pinMode(mainPowerPin, INPUT);
  pinMode(anlagePowerPin, INPUT);

  //set Output pinMode
  pinMode(mainPower, OUTPUT);
  pinMode(summer, OUTPUT);
  pinMode(ledFeuerRed, OUTPUT);
  pinMode(ledAnlageScharf, OUTPUT);
  pinMode(ledAnlagePower, OUTPUT);
  pinMode(ledArduinoPower, OUTPUT);

  //start console
  Serial.begin(9600);
}

/*
   Hauptschleife zur Programmausführung
*/
void loop() {
  read_inputs();
  main_power();
  //prüfe Anlagenpower für Testlauf
  if (anlage_power()) {
    run_test();
    //starte den Feuermelder, sobald die Anlage scharf ist
    if (anlage_scharf()) {
      start_feuermelder();
    }
  }
}

/*
   ließt die Inputs der Schalter
*/
void read_inputs() {
  anlagePowerSchalter = digitalRead(anlagePowerPin);
  anlageScharfSchalter = digitalRead(anlageScharfPin);
  resetTaster = digitalRead(resetPin);
  testTaster = digitalRead(testPin);
  sensorWert = analogRead(sensor);
  mainPowerInput = true;
}

/*
   main-Power ein-/ausschalten
*/
void main_power() {
  //wenn der Schalter auf "an" steht wird main-Power aktiviert
  if ( mainPowerInput) {
    digitalWrite(ledArduinoPower, HIGH);
    digitalWrite(mainPower, HIGH);
    return;
  }
  //wenn der Schalter auf "aus" steht wird das System heruntergefahren
  digitalWrite(ledArduinoPower, LOW);
  digitalWrite(mainPower, LOW);
}

/*
    Strom für die Anlage ein-/ ausschalten
*/
boolean anlage_power() {
  //wenn der Schalter auf "an" steht wird der Anlage Strom zugeführt
  if (anlagePowerSchalter) {
    digitalWrite(ledAnlagePower, HIGH);
    return true;
  }
  //wenn der Schalter auf "aus" steht wird die Anlage heruntergefahren
  digitalWrite(ledAnlagePower, LOW);
  digitalWrite(ledAnlageScharf, LOW);
  //der Alarm wird beendet
  alarm_off();
  //die Abfrage des Testlaufs wird zurückgesetzt
  testDurchgefuhrt = false;
  //Testlauf wird beendet
  runTest = false;
  return false;
}

/*
     Anlage scharf schalten
     Die Anlage darf nur scharf geschalten werden, wenn die Anlage Power hat und der Schalter für "Anlage scharf" auf "an" steht.
     Außerdem muss zuvor ein Testlauf durchgeführt worden sein, es darf aber kein Testlauf aktiv sein
*/
boolean anlage_scharf() {
  //Abfrage, ob nicht alle Bedingungen erfüllt sind
  if (!( anlageScharfSchalter && testDurchgefuhrt && !runTest && anlagePowerSchalter)) {
    digitalWrite(ledAnlageScharf, LOW);
    return false;
  };
  //sind alle Bednigungen erfüllt, wird die Anlage scharf geschalten
  digitalWrite(ledAnlageScharf, HIGH);
  return true;
}

/*
   Startet den Testlauf
*/
void run_test() {
  //start des Testlaufs nur möglich, wenn Anlage nicht scharf
  if (testTaster && !anlage_scharf()) {
    runTest = true;
    digitalWrite(ledAnlagePower, LOW);
    delay(100);
    digitalWrite(ledAnlagePower, HIGH);
    delay(100);
    digitalWrite(ledAnlageScharf, HIGH);
    delay(100);
    alarm_on();
    delay(700);
    digitalWrite(ledAnlagePower, LOW);
    delay(100);
    digitalWrite(ledAnlageScharf, LOW);
    delay(100);
    alarm_off();
    delay(200);
    runTest = false;
    testDurchgefuhrt = true;
  }
}

/*
   aktiviert den Feuermelder
*/
void start_feuermelder() {
  Serial.print("Start:\t");
  Serial.println(sensorWert);
  reset_feueralarm();
  if (sensorWert > 250) {
    alarm_blink = true;
  }
  //startet blinkende visuelle Ausgabe und Sound
  if (alarm_blink) {
    alarm_blinky();
  }
}

/*
    stoppt den Feueralarm
*/
void reset_feueralarm() {
  if (resetTaster) {
    alarm_blink = false;
  }
}

/*
    startet die Alarmsignale
*/
void alarm_on() {
  delay(300);
  digitalWrite(summer, HIGH);
  digitalWrite(ledFeuerRed, HIGH);
  delay(300);
}

/*
    stoppt die Alarmsignale
*/
void alarm_off() {
  digitalWrite(summer, LOW);
  digitalWrite(ledFeuerRed, LOW);
}

/*
    lässt die Alarmsignale blinken
*/
void alarm_blinky() {
  alarm_on();
  alarm_off();
}



